/******************************************************************************/
/* Agent2b - Vending Machine                                                  */
/******************************************************************************/
/* Nome: 8659_VMNSTP_QtVenda_Mapa                                             */
/*                                                                            */
/* Descrio:                                                                 */
/*  ROTINA PARA CLCULO DA QUANTIDADE VENDIDA DO MAPA                         */
/*                                                                            */
/* Criao:                                                                   */
/* 10/07/2003 - ANDR RODRIGO                                                 */
/******************************************************************************/
/* raise_application_error(-20501, '' , True);                                */
/* @C:\Win_V2\Sql\End_Conv\Oracle\8659_VMNSTP_QtVenda_Mapa.SQL                */
/******************************************************************************/
/******************************************************************************/
/* RETORNA A QUANTIDADE DA VENDA DE UM MAPA                                   */
/******************************************************************************/
CREATE OR REPLACE PROCEDURE QTVENDA_MAPA
   (P_CDFILIAL     IN  MAPABASEV.CDFILIAL%TYPE,
    P_NRMAPABASEV  IN  MAPABASEV.NRMAPABASEV%TYPE,
    P_CDITMAPABAS  IN  ITEMAPABASEV.CDITMAPABAS%TYPE,
    P_QTPRODVENDA  OUT ITVENDAVMN.QTPRODVENDA%TYPE,
    P_QTFATUVENDA  OUT ITVENDAVMN.QTFATUVENDA%TYPE,
    P_QTTESTVENDA  OUT ITVENDAVMN.QTTESTVENDA%TYPE,
    P_QTGRANDETOT  OUT MAPABASEV.QTGRANDTOTEV%TYPE,
    P_VRGRANDETOT  OUT MAPABASEV.VRGRANDTOTEV%TYPE,
    P_IDERRO       OUT EQUIPAPROD.IDTPLEIVENDA%TYPE,
    P_QTSALDITMAPA OUT ITEMAPABASEV.QTSALDITMAPA%TYPE)
IS

  V_DTMAPABASEV  MAPABASEV.DTMAPABASEV%TYPE;
  V_CDEQUIPPRO   MAPABASEV.CDEQUIPPRO%TYPE;
  V_IDZERAMAQUI  MAPABASEV.IDZERAMAQUI%TYPE;
  V_CDPRODUTO    ITEMAPABASEV.CDPRODUTO%TYPE;
  V_IDTPLEIVENDA EQUIPAPROD.IDTPLEIVENDA%TYPE;
  V_IDTPAPUMAPA  PARFILIALVMN.IDTPAPUMAPA%TYPE;
  V_QTMAXPROITMA ITEMAPABASEV.QTMAXPROITMA%TYPE;
  V_IDTPPGTOEQUI EQUIPAPROD.IDTPPGTOEQUI%TYPE;
  V_IDZERA       MAPABASEV.IDZERAMAQUI%TYPE;

  V_QTDEESTO       NUMBER;
  V_VALRESTO       NUMBER;
  V_VALRBRUT       NUMBER;
  V_CDFILIMOVI     MAPABASEV.CDFILIAL%TYPE;
  V_CDALMOXARIFE   EQUIPAPROD.CDALMOXARIFE%TYPE;
  V_CDLOCALESTOQ   EQUIPAPROD.CDLOCALESTOQ%TYPE;
  V_EXISTEMAPA_ANT NUMBER;

  V_CTITMAPA       NUMBER;
  V_QTSALDOANT     NUMBER;
  V_QTVEACITANT    NUMBER;
  V_QTFAACITANT    NUMBER;
  V_QTTEACITANT    NUMBER;
  V_QTABASITMAPA   NUMBER;
  V_QTRETIITMAPA   NUMBER;
  V_QTSALDITMAPA   NUMBER;
  V_QTFREPITMAPA   NUMBER;
  V_QTGRANDTOTANT  NUMBER;
  V_VRGRANDTOTANT  NUMBER;
  V_QTGRANDTOTEV   NUMBER;
  V_VRGRANDTOTEV   NUMBER;

  V_QTVEACITMAPA ITEMAPABASEV.QTVEACITMAPA%TYPE;
  V_QTFAACITMAPA ITEMAPABASEV.QTFAACITMAPA%TYPE;
  V_QTTEACITMAPA ITEMAPABASEV.QTTEACITMAPA%TYPE;
BEGIN
   /* P_IDERRO
        A: OK
        B: MOLA NO POSSUI CAPACIDADE MXIMA
        C: ESTOQUE NEGATIVO
        D: QT. ABASTECIDA MAIOR QUE A CAPACIDADE DA MOLA
        E: QT. ABASTECIDA MAIOR QUE A QUANTIDADE NO ESTOQUE
        F: Falta almox/localizacao para a mquina
   */

   SELECT COUNT(*) INTO V_CTITMAPA
   FROM ITEMAPABASEV
   WHERE CDFILIAL    = P_CDFILIAL
     AND NRMAPABASEV = P_NRMAPABASEV
     AND CDITMAPABAS = P_CDITMAPABAS;

   P_IDERRO := 'A';
   IF V_CTITMAPA > 0 THEN
      SELECT MAX(MA.CDEQUIPPRO),   MAX(IT.CDPRODUTO),    MAX(MA.DTMAPABASEV),
             MAX(IT.QTVEACITMAPA), MAX(IT.QTFAACITMAPA), MAX(IT.QTTEACITMAPA),
             MAX(QTFREPITMAPA),    MAX(MA.QTGRANDTOTEV), MAX(MA.VRGRANDTOTEV),
             MAX(QTABASITMAPA),    MAX(QTRETIITMAPA),    MAX(QTSALDITMAPA),
             MAX(IDZERAMAQUI),     MAX(QTMAXPROITMA)
      INTO   V_CDEQUIPPRO,   V_CDPRODUTO,    V_DTMAPABASEV,
             V_QTVEACITMAPA, V_QTFAACITMAPA, V_QTTEACITMAPA,
             V_QTFREPITMAPA, V_QTGRANDTOTEV, V_VRGRANDTOTEV,
             V_QTABASITMAPA, V_QTRETIITMAPA, V_QTSALDITMAPA,
             V_IDZERA      , V_QTMAXPROITMA
      FROM MAPABASEV MA, ITEMAPABASEV IT
      WHERE IT.CDFILIAL    = P_CDFILIAL
        AND IT.NRMAPABASEV = P_NRMAPABASEV
        AND IT.CDITMAPABAS = P_CDITMAPABAS
        AND MA.CDFILIAL    = IT.CDFILIAL
        AND MA.NRMAPABASEV = IT.NRMAPABASEV;

      /*  BUSCA O TIPO DE LEITURA E O TIPO DE PAGAMENTO DA MQUINA */  
      SELECT MAX(E.IDTPLEIVENDA), MAX(E.IDTPPGTOEQUI)
      INTO   V_IDTPLEIVENDA, V_IDTPPGTOEQUI
      FROM   EQUIPAPROD E
      WHERE  E.CDEQUIPPRO = V_CDEQUIPPRO;

      /*  VERIFICA SE EXISTE O PRODUTO EM MAPA ANTERIOR */
      SELECT COUNT(*)
      INTO   V_EXISTEMAPA_ANT
      FROM  MAPABASEV MA, ITEMAPABASEV IT
      WHERE MA.CDFILIAL    = P_CDFILIAL
        AND IT.CDFILIAL    = MA.CDFILIAL
        AND IT.NRMAPABASEV = MA.NRMAPABASEV
        AND IT.CDPRODUTO   = V_CDPRODUTO
        AND MA.CDEQUIPPRO  = V_CDEQUIPPRO
        AND MA.DTMAPABASEV = (SELECT MAX(MB.DTMAPABASEV)
                              FROM   MAPABASEV MB, ITEMAPABASEV IB
                              WHERE IB.CDFILIAL    = P_CDFILIAL
                                AND IB.CDPRODUTO   = V_CDPRODUTO
                                AND MB.CDEQUIPPRO  = V_CDEQUIPPRO
                                AND MB.DTMAPABASEV < V_DTMAPABASEV
                                AND IB.CDFILIAL    = MB.CDFILIAL
                                AND IB.NRMAPABASEV = MB.NRMAPABASEV);

      IF V_EXISTEMAPA_ANT > 0 THEN

         /*  BUSCA A QUANTIDADE DO MAPA ANTERIOR */
         SELECT MAX(IT.QTSALDITMAPA), MAX(QTVEACITMAPA), MAX(QTFAACITMAPA),
                MAX(QTTEACITMAPA),    MAX(IDZERAMAQUI), MAX(QTGRANDTOTEV),
                MAX(VRGRANDTOTEV)
         INTO   V_QTSALDOANT,  V_QTVEACITANT, V_QTFAACITANT,
                V_QTTEACITANT, V_IDZERAMAQUI, V_QTGRANDTOTANT,
                V_VRGRANDTOTANT
         FROM  MAPABASEV MA, ITEMAPABASEV IT
         WHERE MA.CDFILIAL    = P_CDFILIAL
           AND IT.CDFILIAL    = MA.CDFILIAL
           AND IT.NRMAPABASEV = MA.NRMAPABASEV
           AND IT.CDPRODUTO   = V_CDPRODUTO
           AND MA.CDEQUIPPRO  = V_CDEQUIPPRO
           AND MA.DTMAPABASEV = (SELECT MAX(MB.DTMAPABASEV)
                                 FROM   MAPABASEV MB, ITEMAPABASEV IB
                                 WHERE IB.CDFILIAL    = P_CDFILIAL
                                   AND IB.CDPRODUTO   = V_CDPRODUTO
                                   AND MB.CDEQUIPPRO  = V_CDEQUIPPRO
                                   AND MB.DTMAPABASEV < V_DTMAPABASEV
                                   AND IB.CDFILIAL    = MB.CDFILIAL
                                   AND IB.NRMAPABASEV = MB.NRMAPABASEV);
      ELSE
         SELECT MAX(CDFILIMOVI),MAX(CDALMOXARIFE),MAX(CDLOCALESTOQ)
         INTO   V_CDFILIMOVI, V_CDALMOXARIFE, V_CDLOCALESTOQ
         FROM   EQUIPAPROD
         WHERE  CDEQUIPPRO = V_CDEQUIPPRO;

         IF ((V_CDFILIMOVI IS NULL) OR (V_CDALMOXARIFE IS NULL) OR (V_CDLOCALESTOQ IS NULL)) AND (V_IDTPLEIVENDA = '1') THEN
            P_IDErro := 'F';  /* ALMOXARIFADO NO PARAMETRIZADO */
         ELSE
            POSICAO_ESTOQUE(V_CDFILIMOVI,V_CDPRODUTO,V_DTMAPABASEV,V_QTDEESTO,V_VALRESTO,V_VALRBRUT,
                            'N',V_CDALMOXARIFE, V_CDLOCALESTOQ,'S','S','N',NULL,NULL);
         END IF;

         V_QTSALDOANT    := V_QTDEESTO;
         V_QTVEACITANT   := 0;
         V_QTFAACITANT   := 0;
         V_QTTEACITANT   := 0;
         V_IDZERAMAQUI   := '';
         V_QTGRANDTOTANT := 0;
         V_VRGRANDTOTANT := 0;
      END IF;

      P_QTGRANDETOT := NVL(V_QTGRANDTOTEV, 0) - NVL(V_QTGRANDTOTANT, 0);
      P_VRGRANDETOT := NVL(V_VRGRANDTOTEV, 0) - NVL(V_VRGRANDTOTANT, 0);

      P_QTSALDITMAPA := 0;
      IF V_IDTPLEIVENDA = '2' THEN /* LEITURA AUTOMTICA*/
         /* SE MAPA ANTERIOR NAO TIVER ZERADO A MQUINA */
         IF V_IDZERAMAQUI = 'N' THEN
            P_QTPRODVENDA := NVL(V_QTVEACITMAPA, 0) - NVL(V_QTVEACITANT, 0);
            P_QTFATUVENDA := NVL(V_QTFAACITMAPA, 0) - NVL(V_QTFAACITANT, 0);
            P_QTTESTVENDA := NVL(V_QTTEACITMAPA, 0) - NVL(V_QTTEACITANT, 0);
         ELSE
            P_QTPRODVENDA := NVL(V_QTVEACITMAPA, 0);
            P_QTFATUVENDA := NVL(V_QTFAACITMAPA, 0);
            P_QTTESTVENDA := NVL(V_QTTEACITMAPA, 0);
         END IF;
         UPDATE ITEMAPABASEV
         SET    QTVENDITMAPA = P_QTPRODVENDA,
                QTFATUITMAPA = P_QTFATUVENDA,
                QTTESTITMAPA = P_QTTESTVENDA
         WHERE CDFILIAL    = P_CDFILIAL
           AND NRMAPABASEV = P_NRMAPABASEV
           AND CDITMAPABAS = P_CDITMAPABAS;
      ELSE
         SELECT MAX(IDTPAPUMAPA) INTO V_IDTPAPUMAPA
         FROM PARFILIALVMN
         WHERE CDFILIAL = P_CDFILIAL;

         /* CLCULA O ESTOQUE DA MQUINA A PARTIR DA QUANTIDA QUE FALTA A REPOR. */
         IF V_IDTPAPUMAPA = '2' THEN
            IF V_QTMAXPROITMA > 0 THEN
               V_QTSALDITMAPA := V_QTMAXPROITMA - NVL(V_QTFREPITMAPA, 0);
               P_QTSALDITMAPA := V_QTSALDITMAPA;
               
               IF (V_QTSALDITMAPA < 0)  THEN
                  P_IDERRO := 'C';
               END IF;
            ELSE
               P_IDERRO := 'B';
            END IF;

            /* raise_application_error(-20501, 'QT '||V_QTSALDITMAPA||P_IDERRO, True); */
            IF P_IDERRO = 'A' THEN
               UPDATE ITEMAPABASEV
               SET    QTSALDITMAPA = V_QTSALDITMAPA
               WHERE CDFILIAL    = P_CDFILIAL
                 AND NRMAPABASEV = P_NRMAPABASEV
                 AND CDITMAPABAS = P_CDITMAPABAS;
            END IF;
         END IF;

         P_QTFATUVENDA := 0;
         P_QTTESTVENDA := 0;
         P_QTPRODVENDA := 0;
         IF P_IDERRO = 'A' THEN
            /* GRAVA A QUANTIDADE VENDIDA DE ACORDO COM O TIPO DE MQUINA
               1: A VISTA
               2: A PRAZO
               3: AMBOS => NESTE CASO A VENDA A VISTA SER CALCULADA NA CONCILIAO
                           ATRAVS DA SANGRIA
            */
            IF V_IDTPPGTOEQUI = '1' THEN
               P_QTPRODVENDA := NVL(V_QTSALDOANT, 0)   +  /* Saldo Anterior */
                                NVL(V_QTABASITMAPA, 0) -  /* Abastecido     */
                                NVL(V_QTRETIITMAPA, 0) -  /* Retirado       */
                                NVL(V_QTSALDITMAPA, 0);   /* Saldo atual    */

               UPDATE ITEMAPABASEV
               SET    QTVENDITMAPA = P_QTPRODVENDA,
                      QTFATUITMAPA = 0
               WHERE CDFILIAL    = P_CDFILIAL
                 AND NRMAPABASEV = P_NRMAPABASEV
                 AND CDITMAPABAS = P_CDITMAPABAS;
            ELSE
               P_QTFATUVENDA := NVL(V_QTSALDOANT, 0)   +  /* Saldo Anterior */
                                NVL(V_QTABASITMAPA, 0) -  /* Abastecido     */
                                NVL(V_QTRETIITMAPA, 0) -  /* Retirado       */
                                NVL(V_QTSALDITMAPA, 0);   /* Saldo atual    */

               UPDATE ITEMAPABASEV
               SET    QTVENDITMAPA = 0,
                      QTFATUITMAPA = P_QTFATUVENDA
               WHERE CDFILIAL    = P_CDFILIAL
                 AND NRMAPABASEV = P_NRMAPABASEV
                 AND CDITMAPABAS = P_CDITMAPABAS;
            END IF;
         END IF;
      END IF;
   END IF;
END;
/
